#include <msp/io/zlibcompressed.h>
#include "binarywriter.h"
#include "statement.h"
#include "textwriter.h"
#include "writer.h"

using namespace std;

namespace Msp {
namespace DataFile {

Writer::Writer(IO::Base &o):
	out(o),
	mode(new TextWriter(out)),
	binary(false)
{ }

Writer::~Writer()
{
	delete mode;
}

void Writer::write(const Statement &st)
{
	mode->write(st);
}

void Writer::set_binary(bool b)
{
	if(b==binary)
		return;

	binary = b;

	Statement st;
	if(binary)
		st.keyword = "__bin";
	else
		st.keyword = "__text";

	mode->write(st);

	delete mode;
	if(binary)
		mode = new BinaryWriter(out);
	else
		mode = new TextWriter(out);
}

void Writer::set_compressed()
{
	Statement st;
	st.keyword = "__z";
	mode->write(st);

	out.set_compressed();
}

void Writer::set_float_precision(unsigned fp)
{
	mode->set_float_precision(fp);
}

} // namespace DataFile
} // namespace Msp
